@@ -1,13 +1,5 @@
 #ifndef Ny_HEAPY_H
 
-/* Definiing Py_ssize_t for backwards compatibility, from PEP 353 */
-
-#if PY_VERSION_HEX < 0x02050000 && !defined(PY_SSIZE_T_MIN)
-typedef int Py_ssize_t;
-#define PY_SSIZE_T_MAX INT_MAX
-#define PY_SSIZE_T_MIN INT_MIN
-#endif
-
 struct ExtraType;
 
 typedef struct {
@@ -18,17 +10,16 @@ typedef struct {
     PyObject *static_types;
     PyObject *weak_type_callback;
     char is_hiding_calling_interpreter;
-    char is_using_traversing_owner_update;
     struct ExtraType **xt_table;
     int xt_mask;
-    int xt_size;
+    size_t xt_size;
 } NyHeapViewObject;
 
 #define NyHeapView_Check(op) PyObject_TypeCheck(op, &NyHeapView_Type)
 
 typedef struct ExtraType {
     PyTypeObject *xt_type;
-    int (*xt_size) (PyObject *obj);
+    size_t (*xt_size)(PyObject *obj);
     int (*xt_traverse)(struct ExtraType *, PyObject *, visitproc, void *);
     int (*xt_relate)(struct ExtraType *, NyHeapRelate *r);
     struct ExtraType *xt_next;
@@ -37,7 +28,7 @@ typedef struct ExtraType {
     NyHeapViewObject *xt_hv;
     PyObject *xt_weak_type;
     NyHeapDef *xt_hd;
-    long xt_he_offs;
+    Py_ssize_t xt_he_offs;
     int xt_trav_code;
 } ExtraType;
 
@@ -54,4 +45,3 @@ extern PyObject _Ny_RootStateStruct; /* Don't use this directly */
 #define Ny_RootState (&_Ny_RootStateStruct)
 
 #endif /* Ny_HEAPY_H */
-
