@@ -31,6 +31,7 @@ char heapyc_doc[] =
 #include "compile.h"
 #include "frameobject.h"
 #include "../include/guppy.h"
+#include "../include/pythoncapi_compat.h"
 #include "../sets/nodeset.h"
 #include "hpinit.h"
 #include "heapdef.h"
@@ -78,13 +79,6 @@ PyObject *_hiding_tag__name;
 
 #include "impsets.c"
 
-#define VISIT(SLOT)                           \
-    if (SLOT) {                               \
-        err = visit((PyObject *)(SLOT), arg); \
-        if (err)                              \
-            return err;                       \
-    }
-
 static int
 iterable_iterate(PyObject *v, int (*visit)(PyObject *, void *),
                 void *arg)
@@ -161,13 +155,18 @@ err:
 
 /* Other functions */
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 9
 #include "interpreter.c"
+#endif
+
 #include "xmemstats.c"
 
 static PyMethodDef module_methods[] =
 {
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 9
     {"interpreter", (PyCFunction)hp_interpreter, METH_VARARGS, hp_interpreter_doc},
     {"set_async_exc", (PyCFunction)hp_set_async_exc, METH_VARARGS, hp_set_async_exc_doc},
+#endif
     {"xmemstats", (PyCFunction)hp_xmemstats, METH_NOARGS, hp_xmemstats_doc},
     {0}
 };
@@ -202,7 +201,7 @@ NyHeapDef NyHvTypes_HeapDef[] = {
 static struct PyModuleDef moduledef = {
     PyModuleDef_HEAD_INIT,
     MODNAME,
-    NULL,
+    PyDoc_STR(heapyc_doc),
     -1,
     module_methods
 };
@@ -226,7 +225,7 @@ INITFUNC (void)
     PyObject *m = NULL;
     PyObject *d;
 
-    Py_TYPE(&_Ny_RootStateStruct) = &NyRootState_Type;
+    Py_SET_TYPE(&_Ny_RootStateStruct, &NyRootState_Type);
 
     // This has to be here because of 'initializer is not a constant'
     // build error on Windows.
@@ -242,7 +241,6 @@ INITFUNC (void)
         goto error;
     this_module = m;
     d = PyModule_GetDict(m);
-    PyDict_SetItemString(d, "__doc__", PyUnicode_FromString(heapyc_doc));
     PyDict_SetItemString(d, "HeapView",
                          (PyObject *)&NyHeapView_Type);
     PyDict_SetItemString(d, "Horizon",
