@@ -69,8 +69,13 @@ PyDoc_STRVAR(hv_doc,
 "  on the kind of memory allocator, the requested size, etc.\n"
 );
 
-
-
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
+# define Py_BUILD_CORE
+/* PyGC_Head */
+#  undef _PyGC_FINALIZED
+#  include <internal/pycore_gc.h>
+# undef Py_BUILD_CORE
+#endif
 
 #define ALIGN  sizeof(void *)
 #define ALIGN_MASK (ALIGN - 1)
@@ -112,18 +117,17 @@ hv_PyList_Pop(PyObject *list)
 static int
 hv_gc_traverse(NyHeapViewObject *hv, visitproc visit, void *arg)
 {
-    int err;
-    VISIT(hv->root);
-    VISIT(hv->limitframe);
-    VISIT(hv->static_types);
-    VISIT(hv->weak_type_callback);
+    Py_VISIT(hv->root);
+    Py_VISIT(hv->limitframe);
+    Py_VISIT(hv->static_types);
+    Py_VISIT(hv->weak_type_callback);
 
     if (hv->xt_table) {
         size_t i;
         for (i = 0; i < hv->xt_size; i++) {
             ExtraType *xt;
             for (xt = hv->xt_table[i]; xt; xt = xt->xt_next) {
-                VISIT(xt->xt_weak_type);
+                Py_VISIT(xt->xt_weak_type);
             }
         }
     }
@@ -501,9 +505,7 @@ xt_relate(ExtraType *xt, NyHeapRelate *hr)
     if (PyType_Ready(type) == -1)
         return -1;
     if ((PyObject *)type == hr->tgt) {
-/*    if (hr->visit(NYHR_RELSRC, PyUnicode_FromString("type(%s)"), hr)) */
         if (hr->visit(NYHR_INTERATTR, PyUnicode_FromString("ob_type"), hr))
-
             return 0;
     }
     return xt->xt_relate(xt, hr);
@@ -520,6 +522,15 @@ xt_size(ExtraType *xt, PyObject *obj)
 static int
 xt_traverse(ExtraType *xt, PyObject *obj, visitproc visit, void *arg)
 {
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
+    if (Py_TYPE(obj)->tp_flags & Py_TPFLAGS_MANAGED_DICT) {
+        // FIXME: There's no way to distinguish between managed dict entries
+        // and other references. To keep our results stable we have to
+        // materialize this managed dict, which allocates a lot of memory
+        // and will add additional overhead.
+        _PyObject_GetDictPtr(obj);
+    }
+#endif
     if (xt->xt_trav_code == XT_NO)
         return 0;
     else if (xt->xt_trav_code == XT_TP)
@@ -738,10 +749,10 @@ static void
 hv_dealloc(PyObject *v)
 {
     PyObject_GC_UnTrack(v);
-    Py_TRASHCAN_SAFE_BEGIN(v)
+    Py_TRASHCAN_BEGIN(v, hv_dealloc)
     hv_gc_clear((NyHeapViewObject *)v);
     Py_TYPE(v)->tp_free(v);
-    Py_TRASHCAN_SAFE_END(v)
+    Py_TRASHCAN_END
 }
 
 PyDoc_STRVAR(hv_delete_extra_type_doc,
@@ -982,32 +993,19 @@ hv_relate_visit(unsigned int relatype, PyObject *relator, NyHeapRelate *arg_)
 }
 
 typedef struct {
-    NyHeapRelate hr;
+    PyObject *src;
+    PyObject *tgt;
     Py_ssize_t ne;
-    int err;
 }
 NETravArg;
 
-#define NETRAV 1
-
-#if NETRAV
 static int
 hv_ne_rec(PyObject *obj, NETravArg *ta)
 {
-    if (obj == ta->hr.tgt)
+    if (obj == ta->tgt)
         ta->ne++;
     return 0;
 }
-#endif
-
-static int
-hv_ne_visit(unsigned int relatype, PyObject *relator, NyHeapRelate *arg_)
-{
-    NETravArg *ta = (void *)arg_;
-    Py_XDECREF(relator);
-    ta->ne++;
-    return ta->err;
-}
 
 PyDoc_STRVAR(hv_numedges_doc,
 "HV.numedges(src, tgt) -> int\n\
@@ -1018,18 +1016,10 @@ static PyObject *
 hv_numedges(NyHeapViewObject *self, PyObject *args)
 {
     NETravArg ta;
-    if (!PyArg_ParseTuple(args, "OO:numedges", &ta.hr.src, &ta.hr.tgt))
+    if (!PyArg_ParseTuple(args, "OO:numedges", &ta.src, &ta.tgt))
         return NULL;
-    ta.hr.flags = 0;
-    ta.hr.hv = (void *)self;
-    ta.hr.visit = hv_ne_visit;
-    ta.err = 0;
     ta.ne = 0;
-#if NETRAV
-    if (hv_std_traverse(self, ta.hr.src, (visitproc)hv_ne_rec, &ta) == -1)
-#else
-    if (hv_std_relate(&ta.hr) == -1 || ta.err)
-#endif
+    if (hv_std_traverse(self, ta.src, (visitproc)hv_ne_rec, &ta) == -1)
         return 0;
     return PyLong_FromSsize_t(ta.ne);
 }
@@ -1763,6 +1753,7 @@ static PyMethodDef hv_methods[] = {
     {"cli_indisize", (PyCFunction)hv_cli_indisize, METH_VARARGS, hv_cli_indisize_doc},
     {"cli_inrel", (PyCFunction)hv_cli_inrel, METH_VARARGS, hv_cli_inrel_doc},
     {"cli_none", (PyCFunction)hv_cli_none, METH_NOARGS, hv_cli_none_doc},
+    {"cli_prod", (PyCFunction)hv_cli_prod, METH_VARARGS, hv_cli_prod_doc},
     {"cli_rcs", (PyCFunction)hv_cli_rcs, METH_VARARGS, hv_cli_rcs_doc},
     {"cli_type", (PyCFunction)hv_cli_type, METH_NOARGS, hv_cli_type_doc},
     {"cli_user_defined", (PyCFunction)hv_cli_user_defined, METH_VARARGS|METH_KEYWORDS, hv_cli_user_defined_doc},
