@@ -33,67 +33,10 @@ static char hp_set_async_exc_doc[] =
 #include "pythread.h"
 #include "eval.h"
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 8
-# define Py_BUILD_CORE
+#define Py_BUILD_CORE
 /* _PyMem_SetDefaultAllocator */
-#  include <internal/pycore_pymem.h>
-# undef Py_BUILD_CORE
-#elif PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 7
-PyAPI_FUNC(int) _PyMem_SetDefaultAllocator(
-    PyMemAllocatorDomain domain,
-    PyMemAllocatorEx *old_alloc);
-#else
-// source: cpython: Objects/obmalloc.c
-static void *
-_PyMem_RawMalloc(void *ctx, size_t size)
-{
-    if (size == 0)
-        size = 1;
-    return malloc(size);
-}
-
-static void *
-_PyMem_RawCalloc(void *ctx, size_t nelem, size_t elsize)
-{
-    if (nelem == 0 || elsize == 0) {
-        nelem = 1;
-        elsize = 1;
-    }
-    return calloc(nelem, elsize);
-}
-
-static void *
-_PyMem_RawRealloc(void *ctx, void *ptr, size_t size)
-{
-    if (size == 0)
-        size = 1;
-    return realloc(ptr, size);
-}
-
-static void
-_PyMem_RawFree(void *ctx, void *ptr)
-{
-    free(ptr);
-}
-
-static int _PyMem_SetDefaultAllocator(
-    PyMemAllocatorDomain domain,
-    PyMemAllocatorEx *old_alloc)
-{
-    assert(domain == PYMEM_DOMAIN_RAW);
-    PyMem_GetAllocator(domain, old_alloc);
-
-    PyMemAllocatorEx alloc = {
-        .malloc  = _PyMem_RawMalloc,
-        .calloc  = _PyMem_RawCalloc,
-        .realloc = _PyMem_RawRealloc,
-        .free    = _PyMem_RawFree,
-        .ctx     = NULL
-    };
-    PyMem_SetAllocator(domain, &alloc);
-    return 0;
-}
-#endif
+# include <internal/pycore_pymem.h>
+#undef Py_BUILD_CORE
 
 struct bootstate {
     PyObject *cmd;
@@ -196,9 +139,8 @@ t_bootstrap(void *boot_raw)
     }
 
     Py_EndInterpreter(tstate);
-    // Not deprecated in code because t_bootstrap in _thread module still uses
-    // it. If they have a solution for this issue of trying to release GIL while
-    // current thread state is NULL we can use their solution.
+    // We can't use _PyEval_ReleaseLock(NULL) here on Py3.9+
+    // because of fvisibility=hidden.
     PyEval_ReleaseLock();
     PyThread_exit_thread();
 }
@@ -227,7 +169,9 @@ hp_interpreter(PyObject *self, PyObject *args)
     Py_INCREF(cmd);
     Py_XINCREF(locals);
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 9
     PyEval_InitThreads(); // Start the interpreter's thread-awareness
+#endif
 
     evt_ready = PyThread_allocate_lock();
     if (evt_ready == NULL) {
@@ -265,7 +209,7 @@ Err:
 */
 
 
-Py_ssize_t
+static Py_ssize_t
 NyThreadState_SetAsyncExc(long id, PyObject *exc) {
     PyInterpreterState *interp;
     Py_ssize_t count = 0;
