@@ -103,11 +103,16 @@ char rootstate_doc[] =
 "that is the number of frames it has before it in call order.\n"
 ;
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 8
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION == 8
 # define Py_BUILD_CORE
 /* PyInterpreterState */
 #  include <internal/pycore_pystate.h>
 # undef Py_BUILD_CORE
+#elif PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
+# define Py_BUILD_CORE
+/* PyInterpreterState */
+#  include <internal/pycore_interp.h>
+# undef Py_BUILD_CORE
 #endif
 
 #define THREAD_ID(ts)    (ts->thread_id)
@@ -130,95 +135,87 @@ rootstate_dealloc(void *arg)
 
 
 #define MEMBER(name) {#name, T_OBJECT, offsetof(PyInterpreterState, name)}
+#define RENAMEMEMBER(name, newname) {#newname, T_OBJECT, offsetof(PyInterpreterState, name)}
 
 static struct PyMemberDef is_members[] = {
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    RENAMEMEMBER(imports.modules, modules),
+    RENAMEMEMBER(imports.modules_by_index, modules_by_index),
+    RENAMEMEMBER(imports.importlib, importlib),
+    RENAMEMEMBER(imports.import_func, import_func),
+#else
     MEMBER(modules),
     MEMBER(modules_by_index),
+    MEMBER(importlib),
+    MEMBER(import_func),
+#endif
+
     MEMBER(sysdict),
     MEMBER(builtins),
-    MEMBER(importlib),
 
     MEMBER(codec_search_path),
     MEMBER(codec_search_cache),
     MEMBER(codec_error_registry),
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 8
     MEMBER(dict),
-#endif
-
     MEMBER(builtins_copy),
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 6
-    MEMBER(import_func),
-#endif
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 7
 #ifdef HAVE_FORK
     MEMBER(before_forkers),
     MEMBER(after_forkers_parent),
     MEMBER(after_forkers_child),
 #endif
-
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 10
     MEMBER(pyexitmodule),
 #endif
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 8
     MEMBER(audit_hooks),
-#endif
     {0} /* Sentinel */
 };
 
 #undef MEMBER
+#undef RENAMEMEMBER
 
 #define MEMBER(name) {#name, T_OBJECT, offsetof(PyThreadState, name)}
 #define RENAMEMEMBER(name, newname) {#newname, T_OBJECT, offsetof(PyThreadState, name)}
 
 static struct PyMemberDef ts_members[] = {
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 11
     MEMBER(frame),
+#endif
+
     MEMBER(c_profileobj),
     MEMBER(c_traceobj),
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+    MEMBER(current_exception),
+#else
     MEMBER(curexc_type),
     MEMBER(curexc_value),
     MEMBER(curexc_traceback),
+#endif
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 7
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
+    RENAMEMEMBER(exc_state.exc_value, exc_value),
+#else
     RENAMEMEMBER(exc_state.exc_type, exc_type),
     RENAMEMEMBER(exc_state.exc_value, exc_value),
     RENAMEMEMBER(exc_state.exc_traceback, exc_traceback),
-#else
-    MEMBER(exc_type),
-    MEMBER(exc_value),
-    MEMBER(exc_traceback),
 #endif
 
     MEMBER(dict),
     MEMBER(async_exc),
     // trash_delete_later not included
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 8
-    MEMBER(coroutine_wrapper),
-#endif
-
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 6
     MEMBER(async_gen_firstiter),
     MEMBER(async_gen_finalizer),
-#endif
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 7
     MEMBER(context),
-#endif
     {0} /* Sentinel */
 };
 
 #undef MEMBER
-
-
-#define VISIT(SLOT)                           \
-    if (SLOT) {                               \
-        err = visit((PyObject *)(SLOT), arg); \
-        if (err)                              \
-            return err;                       \
-    }
+#undef RENAMEMEMBER
 
 #define ISATTR(name)                                                                  \
     if ((PyObject *)is->name == r->tgt) {                                             \
@@ -226,6 +223,12 @@ static struct PyMemberDef ts_members[] = {
             return 1;                                                                 \
     }
 
+#define RENAMEISATTR(name, newname)                                                                  \
+    if ((PyObject *)is->name == r->tgt) {                                             \
+        if (r->visit(NYHR_ATTRIBUTE, PyUnicode_FromFormat("i%d_%s", isno, #newname), r)) \
+            return 1;                                                                 \
+    }
+
 #define TSATTR(name)                                                                                      \
     if ((PyObject *)ts->name == r->tgt) {                                                                 \
         if (r->visit(NYHR_ATTRIBUTE, PyUnicode_FromFormat("i%d_t%lu_%s", isno, THREAD_ID(ts), #name), r)) \
@@ -273,50 +276,71 @@ rootstate_relate(NyHeapRelate *r)
     for (is = PyInterpreterState_Head(), isno--;
          is;
          is = PyInterpreterState_Next(is), isno--) {
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
+        if (is != PyInterpreterState_Get())
+            continue;
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        RENAMEISATTR(imports.modules, modules);
+        RENAMEISATTR(imports.modules_by_index, modules_by_index);
+        RENAMEISATTR(imports.importlib, importlib);
+        RENAMEISATTR(imports.import_func, import_func);
+#else
         ISATTR(modules);
         ISATTR(modules_by_index);
+        ISATTR(importlib);
+        ISATTR(import_func);
+#endif
+
         ISATTR(sysdict);
         ISATTR(builtins);
-        ISATTR(importlib);
 
         ISATTR(codec_search_path);
         ISATTR(codec_search_cache);
         ISATTR(codec_error_registry);
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 8
         ISATTR(dict);
-#endif
-
         ISATTR(builtins_copy);
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 6
-        ISATTR(import_func);
-#endif
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 7
-# ifdef HAVE_FORK
+#ifdef HAVE_FORK
         ISATTR(before_forkers);
         ISATTR(after_forkers_parent);
         ISATTR(after_forkers_child);
-# endif
-
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 10
         ISATTR(pyexitmodule);
 #endif
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 8
         ISATTR(audit_hooks);
-#endif
 
-        for (ts = is->tstate_head; ts; ts = ts->next) {
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
+        ts = is->threads.head;
+#else
+        ts = is->tstate_head;
+#endif
+        for (; ts; ts = ts->next) {
             if ((ts == bts && r->tgt == hv->limitframe) ||
                     (!hv->limitframe && isframe)) {
                 int frameno = -1;
                 int numframes = 0;
                 PyFrameObject *frame;
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
+                for (frame = PyThreadState_GetFrame(ts); frame;) {
+                    PyFrameObject *next_frame = PyFrame_GetBack(frame);
+                    numframes++;
+                    if ((PyObject *)frame == r->tgt)
+                        frameno = numframes;
+
+                    Py_DECREF(frame);
+                    frame = next_frame;
+                }
+#else
                 for (frame = (PyFrameObject *)ts->frame; frame; frame = frame->f_back) {
                     numframes++;
-                    if (r->tgt == (PyObject *)frame)
+                    if ((PyObject *)frame == r->tgt)
                         frameno = numframes;
                 }
+#endif
                 if (frameno != -1) {
                     frameno = numframes - frameno;
                     if (r->visit(NYHR_ATTRIBUTE, PyUnicode_FromFormat("i%d_t%lu_f%d", isno, THREAD_ID(ts), frameno), r))
@@ -326,35 +350,29 @@ rootstate_relate(NyHeapRelate *r)
             TSATTR(c_profileobj);
             TSATTR(c_traceobj);
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+            TSATTR(current_exception);
+#else
             TSATTR(curexc_type);
             TSATTR(curexc_value);
             TSATTR(curexc_traceback);
+#endif
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 7
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
+            RENAMETSATTR(exc_state.exc_value, exc_value);
+#else
             RENAMETSATTR(exc_state.exc_type, exc_type);
             RENAMETSATTR(exc_state.exc_value, exc_value);
             RENAMETSATTR(exc_state.exc_traceback, exc_traceback);
-#else
-            TSATTR(exc_type);
-            TSATTR(exc_value);
-            TSATTR(exc_traceback);
 #endif
 
             TSATTR(dict);
             TSATTR(async_exc);
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 8
-            TSATTR(coroutine_wrapper);
-#endif
-
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 6
             TSATTR(async_gen_firstiter);
             TSATTR(async_gen_finalizer);
-#endif
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 7
             TSATTR(context);
-#endif
         }
     }
     return 0;
@@ -369,84 +387,92 @@ rootstate_traverse(NyHeapTraverse *ta)
     void *arg = ta->arg;
     PyThreadState *ts, *bts = PyThreadState_GET();
     PyInterpreterState *is;
-    int err;
 
     for (is = PyInterpreterState_Head(); is; is = PyInterpreterState_Next(is)) {
         if (hv->is_hiding_calling_interpreter && is == bts->interp)
             continue;
-        VISIT(is->modules);
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
+        if (is != PyInterpreterState_Get())
+            continue;
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+        Py_VISIT(is->imports.modules);
         // Not traversing through this because it is of the same level as
         // modules, making pathfinding generate an extra path.
-        // VISIT(is->modules_by_index);
-        VISIT(is->sysdict);
-        VISIT(is->builtins);
-        VISIT(is->importlib);
-
-        VISIT(is->codec_search_path);
-        VISIT(is->codec_search_cache);
-        VISIT(is->codec_error_registry);
-
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 8
-        VISIT(is->dict);
-#endif
-
-        VISIT(is->builtins_copy);
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 6
-        VISIT(is->import_func);
-#endif
-
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 7
-# ifdef HAVE_FORK
-        VISIT(is->before_forkers);
-        VISIT(is->after_forkers_parent);
-        VISIT(is->after_forkers_child);
-# endif
-
-        VISIT(is->pyexitmodule);
-#endif
-
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 8
-        VISIT(is->audit_hooks);
-#endif
-
-        for (ts = is->tstate_head; ts; ts = ts->next) {
-            if (ts == bts && hv->limitframe) {
-                VISIT(hv->limitframe);
-            } else if (!hv->limitframe) {
-                VISIT(ts->frame);
-            }
-            VISIT(ts->c_profileobj);
-            VISIT(ts->c_traceobj);
-
-            VISIT(ts->curexc_type);
-            VISIT(ts->curexc_value);
-            VISIT(ts->curexc_traceback);
-
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 7
-            VISIT(ts->exc_state.exc_type);
-            VISIT(ts->exc_state.exc_value);
-            VISIT(ts->exc_state.exc_traceback);
+        // Py_VISIT(is->imports.modules_by_index);
+        Py_VISIT(is->imports.importlib);
+        Py_VISIT(is->imports.import_func);
 #else
-            VISIT(ts->exc_type);
-            VISIT(ts->exc_value);
-            VISIT(ts->exc_traceback);
+        Py_VISIT(is->modules);
+        // Py_VISIT(is->modules_by_index);
+        Py_VISIT(is->importlib);
+        Py_VISIT(is->import_func);
 #endif
 
-            VISIT(ts->dict);
-            VISIT(ts->async_exc);
+        Py_VISIT(is->sysdict);
+        Py_VISIT(is->builtins);
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 8
-            VISIT(ts->coroutine_wrapper);
+        Py_VISIT(is->codec_search_path);
+        Py_VISIT(is->codec_search_cache);
+        Py_VISIT(is->codec_error_registry);
+
+        Py_VISIT(is->dict);
+        Py_VISIT(is->builtins_copy);
+
+#ifdef HAVE_FORK
+        Py_VISIT(is->before_forkers);
+        Py_VISIT(is->after_forkers_parent);
+        Py_VISIT(is->after_forkers_child);
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 10
+        Py_VISIT(is->pyexitmodule);
 #endif
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 6
-            VISIT(ts->async_gen_firstiter);
-            VISIT(ts->async_gen_finalizer);
+        Py_VISIT(is->audit_hooks);
+
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
+        ts = is->threads.head;
+#else
+        ts = is->tstate_head;
+#endif
+        for (; ts; ts = ts->next) {
+            if (ts == bts && hv->limitframe) {
+                Py_VISIT(hv->limitframe);
+            } else if (!hv->limitframe) {
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
+                PyFrameObject *frame = PyThreadState_GetFrame(ts);
+                Py_VISIT(frame);
+                Py_XDECREF(frame);
+#else
+                Py_VISIT(ts->frame);
+#endif
+            }
+            Py_VISIT(ts->c_profileobj);
+            Py_VISIT(ts->c_traceobj);
+
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+            Py_VISIT(ts->current_exception);
+#else
+            Py_VISIT(ts->curexc_type);
+            Py_VISIT(ts->curexc_value);
+            Py_VISIT(ts->curexc_traceback);
 #endif
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 7
-            VISIT(ts->context);
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
+            Py_VISIT(ts->exc_state.exc_value);
+#else
+            Py_VISIT(ts->exc_state.exc_type);
+            Py_VISIT(ts->exc_state.exc_value);
+            Py_VISIT(ts->exc_state.exc_traceback);
 #endif
+
+            Py_VISIT(ts->dict);
+            Py_VISIT(ts->async_exc);
+
+            Py_VISIT(ts->async_gen_firstiter);
+            Py_VISIT(ts->async_gen_finalizer);
+
+            Py_VISIT(ts->context);
         }
     }
     return 0;
@@ -473,6 +499,7 @@ rootstate_getattr(PyObject *obj, PyObject *name)
 {
     const char *s = PyUnicode_AsUTF8(name);
     PyInterpreterState *is;
+    PyThreadState *ts;
     int n = 0;
     int ino;
     unsigned long tno;
@@ -489,28 +516,59 @@ rootstate_getattr(PyObject *obj, PyObject *name)
         for (is = PyInterpreterState_Head(), countis = 0;
              is;
              is = PyInterpreterState_Next(is), countis++) {
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
+            if (is != PyInterpreterState_Get())
+                continue;
+#endif
             int isno = numis - countis - 1;
             if (isno == ino) {
                 if (sscanf(s, "t%lu_%n", &tno, &n) == 1) {
                     s += n;
-                    PyThreadState *ts;
-                    for (ts = is->tstate_head; ts; ts = ts->next) {
+
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
+                    ts = is->threads.head;
+#else
+                    ts = is->tstate_head;
+#endif
+                    for (; ts; ts = ts->next) {
                         if (THREAD_ID(ts) == tno) {
                             int frameno = 0;
                             if (sscanf(s, "f%d%n", &frameno, &n) == 1 && s[n] == '\0') {
-                                PyFrameObject *frame;
                                 int numframes = 0;
+                                PyFrameObject *frame;
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
+                                PyFrameObject *current_frame = PyThreadState_GetFrame(ts);
+                                for (frame = (PyFrameObject *)Py_XNewRef(current_frame); frame;) {
+                                    PyFrameObject *next_frame = PyFrame_GetBack(frame);
+                                    numframes++;
+                                    Py_DECREF(frame);
+                                    frame = next_frame;
+                                }
+                                for (frame = (PyFrameObject *)Py_XNewRef(current_frame); frame;) {
+                                    PyFrameObject *next_frame = PyFrame_GetBack(frame);
+                                    numframes--;
+                                    if (numframes == frameno) {
+                                        Py_DECREF(current_frame);
+                                        Py_DECREF(name);
+                                        return (PyObject *)frame;
+                                    }
+                                    Py_DECREF(frame);
+                                    frame = next_frame;
+                                }
+                                Py_DECREF(current_frame);
+#else
                                 for (frame = ts->frame; frame; frame = frame->f_back) {
-                                    numframes ++;
+                                    numframes++;
                                 }
                                 for (frame = ts->frame; frame; frame = frame->f_back) {
-                                    numframes --;
+                                    numframes--;
                                     if (numframes == frameno) {
                                         Py_INCREF(frame);
                                         Py_DECREF(name);
                                         return (PyObject *)frame;
                                     }
                                 }
+#endif
                                 PyErr_Format(PyExc_AttributeError,
                                              "thread state has no frame numbered %d from bottom",
                                              frameno);
@@ -555,9 +613,18 @@ rootstate_getattr(PyObject *obj, PyObject *name)
         for (is = PyInterpreterState_Head(), countis = 0;
              is;
              is = PyInterpreterState_Next(is), countis++) {
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
+            if (is != PyInterpreterState_Get())
+                continue;
+#endif
             int isno = numis - countis - 1;
-            PyThreadState *ts;
-            for (ts = is->tstate_head; ts; ts = ts->next) {
+
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
+            ts = is->threads.head;
+#else
+            ts = is->tstate_head;
+#endif
+            for (; ts; ts = ts->next) {
                 if (THREAD_ID(ts) == tno) {
                     PyObject *fullname = PyUnicode_FromFormat("i%d_%U", isno, name);
                     if (!fullname) {
@@ -612,82 +679,94 @@ rootstate_dir(PyObject *self, PyObject *args)
     for (is = PyInterpreterState_Head(), isno--;
          is;
          is = PyInterpreterState_Next(is), isno--) {
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 9
+        if (is != PyInterpreterState_Get())
+            continue;
+#endif
         ISATTR_DIR(modules);
         ISATTR_DIR(modules_by_index);
+        ISATTR_DIR(importlib);
+        ISATTR_DIR(import_func);
+
         ISATTR_DIR(sysdict);
         ISATTR_DIR(builtins);
-        ISATTR_DIR(importlib);
 
         ISATTR_DIR(codec_search_path);
         ISATTR_DIR(codec_search_cache);
         ISATTR_DIR(codec_error_registry);
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 8
         ISATTR_DIR(dict);
-#endif
-
         ISATTR_DIR(builtins_copy);
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 6
-        ISATTR_DIR(import_func);
-#endif
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 7
-# ifdef HAVE_FORK
+#ifdef HAVE_FORK
         ISATTR_DIR(before_forkers);
         ISATTR_DIR(after_forkers_parent);
         ISATTR_DIR(after_forkers_child);
-# endif
-
+#endif
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 10
         ISATTR_DIR(pyexitmodule);
 #endif
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 8
         ISATTR_DIR(audit_hooks);
-#endif
 
-        for (ts = is->tstate_head; ts; ts = ts->next) {
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
+        ts = is->threads.head;
+#else
+        ts = is->tstate_head;
+#endif
+        for (; ts; ts = ts->next) {
             int numframes = 0;
             PyFrameObject *frame;
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 11
+            for (frame = PyThreadState_GetFrame(ts); frame;) {
+                PyFrameObject *next_frame = PyFrame_GetBack(frame);
+                numframes++;
+                Py_DECREF(frame);
+                frame = next_frame;
+            }
+#else
             for (frame = (PyFrameObject *)ts->frame; frame; frame = frame->f_back) {
                 numframes++;
             }
+#endif
             int frameno;
             for (frameno = 0; frameno < numframes; frameno++) {
                 attr = PyUnicode_FromFormat("i%d_t%lu_f%d", isno, THREAD_ID(ts), frameno);
-                if (attr) {
-                    if (PyList_Append(list, attr)) {
-                        Py_DECREF(attr);
-                        goto Err;
-                    }
+                if (!attr)
+                    goto Err;
+
+                if (PyList_Append(list, attr)) {
                     Py_DECREF(attr);
+                    goto Err;
                 }
+                Py_DECREF(attr);
             }
             TSATTR_DIR(c_profileobj);
             TSATTR_DIR(c_traceobj);
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+            TSATTR_DIR(current_exception);
+#else
             TSATTR_DIR(curexc_type);
             TSATTR_DIR(curexc_value);
             TSATTR_DIR(curexc_traceback);
+#endif
 
+#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 12
+            TSATTR_DIR(exc_value);
+#else
             TSATTR_DIR(exc_type);
             TSATTR_DIR(exc_value);
             TSATTR_DIR(exc_traceback);
+#endif
 
             TSATTR_DIR(dict);
             TSATTR_DIR(async_exc);
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION < 8
-            TSATTR_DIR(coroutine_wrapper);
-#endif
-
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 6
             TSATTR_DIR(async_gen_firstiter);
             TSATTR_DIR(async_gen_finalizer);
-#endif
 
-#if PY_MAJOR_VERSION >= 3 && PY_MINOR_VERSION >= 7
             TSATTR_DIR(context);
-#endif
         }
     }
 
